from typing import List

import numpy as np

IDENTITY_MATRIX_4D = [1., 0., 0., 0.,
                      0., 1., 0., 0.,
                      0., 0., 1., 0.,
                      0., 0., 0., 1.]


def calculate_boundary(array: np.ndarray) -> np.ndarray:
    width = array.shape[0]
    height = array.shape[1]
    xbig = np.expand_dims(np.array(range(width)), -1).repeat(height, axis=1)
    ybig = np.expand_dims(np.array(range(height)), 0).repeat(width, axis=0)
    cond = array == 0

    aabb = []
    xbig[cond] = np.iinfo(np.intc).max
    ybig[cond] = np.iinfo(np.intc).max
    aabb.append(np.min(xbig))
    aabb.append(np.min(ybig))
    xbig[cond] = 0
    ybig[cond] = 0
    aabb.append(np.max(xbig))
    aabb.append(np.max(ybig))
    return aabb


def get_smoothed_pixel(data: np.ndarray, x: int, y: int, step: int) -> np.array:
    width = data.shape[0]
    height = data.shape[1]
    pixel = np.array([0, 0, 0])
    count = 0
    for tx in range(x - step, x + step):
        for ty in range(y - step, y + step):
            if not (0 < tx < width and 0 < ty < height):
                continue
            pixel = pixel + data[tx, ty, 0]
            count = count + 1
    return pixel / max(count, 1)


def matrix_calculate(position: List[float], rotation: List[float]) -> List[float]:
    """Calculate a matrix image->world from device position and rotation"""

    output = IDENTITY_MATRIX_4D

    sqw = rotation[3] * rotation[3]
    sqx = rotation[0] * rotation[0]
    sqy = rotation[1] * rotation[1]
    sqz = rotation[2] * rotation[2]

    invs = 1 / (sqx + sqy + sqz + sqw)
    output[0] = (sqx - sqy - sqz + sqw) * invs
    output[5] = (-sqx + sqy - sqz + sqw) * invs
    output[10] = (-sqx - sqy + sqz + sqw) * invs

    tmp1 = rotation[0] * rotation[1]
    tmp2 = rotation[2] * rotation[3]
    output[1] = 2.0 * (tmp1 + tmp2) * invs
    output[4] = 2.0 * (tmp1 - tmp2) * invs

    tmp1 = rotation[0] * rotation[2]
    tmp2 = rotation[1] * rotation[3]
    output[2] = 2.0 * (tmp1 - tmp2) * invs
    output[8] = 2.0 * (tmp1 + tmp2) * invs

    tmp1 = rotation[1] * rotation[2]
    tmp2 = rotation[0] * rotation[3]
    output[6] = 2.0 * (tmp1 + tmp2) * invs
    output[9] = 2.0 * (tmp1 - tmp2) * invs

    output[12] = -position[0]
    output[13] = -position[1]
    output[14] = -position[2]
    return output


def matrix_transform_point(point: np.ndarray, device_pose_arr: np.ndarray) -> np.ndarray:
    """Transformation of point by device pose matrix

    point(np.array of float): 3D point
    device_pose: flattened 4x4 matrix

    Returns:
        3D point(np.array of float)
    """
    point_4d = np.append(point, 1.)
    output = np.matmul(device_pose_arr, point_4d)
    output[0:2] = output[0:2] / abs(output[3])
    return output[0:-1]


def parse_numbers(line: str) -> List[float]:
    """Parse line of numbers

    Args:
        line: Example: "0.6786797 0.90489584 0.49585155 0.5035042"

    Return:
        numbers: [0.6786797, 0.90489584, 0.49585155, 0.5035042]
    """
    return [float(value) for value in line.split(' ')]



import logging
import logging.config
import os
from pathlib import Path
import time
from typing import List, Tuple

import glob2 as glob
import numpy as np
import pandas as pd
import tensorflow as tf
from tensorflow.keras.models import load_model
from azureml.core import Workspace
from bunch import Bunch

from cgmml.common.model_utils.preprocessing_multiartifact_python import create_multiartifact_paths_for_qrcodes
from cgmml.common.model_utils.preprocessing_multiartifact_tensorflow import create_multiartifact_sample
from .constants_eval import (
    COLUMN_NAME_AGE, COLUMN_NAME_GOODBAD, COLUMN_NAME_SEX,
    GOODBAD_IDX, GOODBAD_DICT, SEX_IDX, AGE_IDX, HEIGHT_IDX, WEIGHT_IDX)
from .eval_utils import (
    avgerror, calculate_performance)
from .uncertainty_utils import get_prediction_uncertainty_deepensemble
from .eval_utilities import (
    download_model, get_model_path, get_depthmap_files, filter_dataset_according_to_standing_lying, tf_load_pickle,
    get_prediction, calculate_and_save_results, calculate_performance_age,
    draw_age_scatterplot, draw_stunting_diagnosis, draw_wasting_diagnosis,
    calculate_performance_sex, calculate_performance_goodbad, get_predictions_from_multiple_models,
    draw_uncertainty_goodbad_plot, get_prediction_multiartifact, get_column_list, prepare_sample_dataset,
    draw_uncertainty_scatterplot, is_filter_enabled)

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s - %(pathname)s: line %(lineno)d'))
logger.addHandler(handler)


class Evaluation:
    def __init__(self, model_config: Bunch, data_config: Bunch, model_base_dir: Path, dataset_path: str):
        self.model_config = model_config
        self.data_config = data_config
        self.model_base_dir = model_base_dir
        self.dataset_path = dataset_path
        self._input_location = os.path.join(self.model_config.INPUT_LOCATION, self.model_config.NAME)

    def get_the_model_path(self, workspace: Workspace):
        logger.info(f"Model will download from '{self._input_location}' to '{self.model_base_dir}'")
        download_model(workspace=workspace,
                       experiment_name=self.model_config.EXPERIMENT_NAME,
                       run_id=self.model_config.RUN_ID,
                       input_location=self._input_location,
                       output_location=self.model_base_dir)
        logger.info("Model was downloaded")
        self.model_path = self.model_base_dir / get_model_path(self.model_config)
        self.model_path_or_paths = self.model_path

    def get_the_qr_code_path(self) -> List[str]:
        dataset_path = os.path.join(self.dataset_path, "scans")
        logger.info('Dataset path: %s', dataset_path)
        logger.info('Getting QR-code paths...')
        qrcode_paths = glob.glob(os.path.join(dataset_path, "*"))
        logger.info('qrcode_paths: %d', len(qrcode_paths))
        assert len(qrcode_paths) != 0

        logger.info('Paths for evaluation: \n\t' + '\n\t'.join(qrcode_paths))
        logger.info(len(qrcode_paths))
        return qrcode_paths

    def prepare_dataset(self,
                        qrcode_paths: List[str],
                        filter_config: Bunch) -> Tuple[tf.data.Dataset, List[str]]:
        # Get depthmaps
        logger.info("Getting Depthmap paths...")
        paths_evaluation = get_depthmap_files(qrcode_paths)
        del qrcode_paths

        logger.info("Using %d artifact files for evaluation.", len(paths_evaluation))

        paths_belonging_to_predictions = paths_evaluation

        if is_filter_enabled(filter_config):
            standing_model = load_model(filter_config.NAME)
            paths_belonging_to_predictions = filter_dataset_according_to_standing_lying(paths_evaluation,
                                                                                        standing_model)
        logger.info("Creating dataset for training.")
        paths = paths_belonging_to_predictions
        dataset = tf.data.Dataset.from_tensor_slices(paths)
        dataset_norm = dataset.map(
            lambda path: tf_load_pickle(path, self.data_config.NORMALIZATION_VALUE, self.data_config)
        )

        # filter goodbad==delete
        if GOODBAD_IDX in self.data_config.TARGET_INDEXES:
            goodbad_index = self.data_config.TARGET_INDEXES.index(GOODBAD_IDX)
            dataset_norm = dataset_norm.filter(
                lambda _path, _depthmap, targets: targets[goodbad_index] != GOODBAD_DICT['delete'])

        dataset_norm = dataset_norm.cache()
        dataset_norm = dataset_norm.prefetch(tf.data.experimental.AUTOTUNE)
        temp_dataset_evaluation = dataset_norm
        del dataset_norm
        logger.info("Created dataset for training.")

        # Update paths_belonging_to_predictions after filtering
        dataset_paths = temp_dataset_evaluation.map(lambda path, _depthmap, _targets: path)
        list_paths = list(dataset_paths.as_numpy_iterator())
        paths_belonging_to_predictions = [x.decode() for x in list_paths]

        dataset_evaluation = temp_dataset_evaluation.map(lambda _path, depthmap, targets: (depthmap, targets))
        del temp_dataset_evaluation
        return dataset_evaluation, paths_belonging_to_predictions

    def get_prediction_(self, model_path: Path, dataset_evaluation: tf.data.Dataset) -> np.ndarray:
        return get_prediction(model_path, dataset_evaluation, self.data_config)

    def prepare_dataframe(self,
                          paths_belonging_to_predictions: List[str],
                          prediction_array: np.ndarray,
                          result_config: Bunch) -> pd.DataFrame:
        qrcode_list, scantype_list, artifact_list, prediction_list, target_list = get_column_list(
            paths_belonging_to_predictions, prediction_array, self.data_config)

        df = pd.DataFrame({
            'qrcode': qrcode_list,
            'artifact': artifact_list,
            'scantype': scantype_list,
            'GT': target_list if target_list[0].shape == tuple() else [el[0] for el in target_list],
            'predicted': prediction_list
        }, columns=result_config.COLUMNS)
        df['GT'] = df['GT'].astype('float64')
        df['predicted'] = df['predicted'].astype('float64')

        if 'AGE_BUCKETS' in result_config.keys():
            idx = self.data_config.TARGET_INDEXES.index(AGE_IDX)
            df[COLUMN_NAME_AGE] = [el[idx] for el in target_list]
        if SEX_IDX in self.data_config.TARGET_INDEXES:
            idx = self.data_config.TARGET_INDEXES.index(SEX_IDX)
            df[COLUMN_NAME_SEX] = [el[idx] for el in target_list]
        if GOODBAD_IDX in self.data_config.TARGET_INDEXES:
            idx = self.data_config.TARGET_INDEXES.index(GOODBAD_IDX)
            df[COLUMN_NAME_GOODBAD] = [el[idx] for el in target_list]

        logger.info("df.shape: %s", df.shape)
        return df

    def evaluate(self,
                 df: pd.DataFrame,
                 result_config: Bunch,
                 eval_config: Bunch,
                 OUTPUT_CSV_PATH: str,
                 descriptor: str):
        df_grouped = df.groupby(['qrcode', 'scantype']).mean()
        logger.info("Mean Avg Error: %s", df_grouped)

        df_grouped['error'] = df_grouped.apply(avgerror, axis=1)

        csv_fpath = f"{OUTPUT_CSV_PATH}/{descriptor}.csv"
        logger.info("Calculate and save the results to %s", csv_fpath)
        calculate_and_save_results(df_grouped, eval_config.NAME, csv_fpath,
                                   self.data_config, result_config, fct=calculate_performance)

        sample_csv_fpath = f"{OUTPUT_CSV_PATH}/inaccurate_scans_{descriptor}.csv"
        df_grouped.to_csv(sample_csv_fpath, index=True)

        if 'AGE_BUCKETS' in result_config.keys():
            csv_fpath = f"{OUTPUT_CSV_PATH}/age_evaluation_{descriptor}.csv"
            logger.info("Calculate and save age results to %s", csv_fpath)
            calculate_and_save_results(df_grouped, eval_config.NAME, csv_fpath,
                                       self.data_config, result_config, fct=calculate_performance_age)
            png_fpath = f"{OUTPUT_CSV_PATH}/age_evaluation_scatter_{descriptor}.png"
            logger.info("Calculate and save scatterplot results to %s", png_fpath)
            draw_age_scatterplot(df, png_fpath)

        if (HEIGHT_IDX in self.data_config.TARGET_INDEXES
                and AGE_IDX in self.data_config.TARGET_INDEXES
                and descriptor != self.model_config.EXPERIMENT_NAME):
            png_fpath = f"{OUTPUT_CSV_PATH}/stunting_diagnosis_{descriptor}.png"
            logger.info("Calculate zscores and save confusion matrix results to %s", png_fpath)
            start = time.time()
            draw_stunting_diagnosis(df, png_fpath)
            end = time.time()
            logger.info("Total time for Calculate zscores and save confusion matrix: %.2f", end - start)

        if (WEIGHT_IDX in self.data_config.TARGET_INDEXES
                and AGE_IDX in self.data_config.TARGET_INDEXES
                and descriptor != self.model_config.EXPERIMENT_NAME):
            png_fpath = f"{OUTPUT_CSV_PATH}/wasting_diagnosis_{descriptor}.png"
            logger.info("Calculate and save wasting confusion matrix results to %s", png_fpath)
            start = time.time()
            draw_wasting_diagnosis(df, png_fpath)
            end = time.time()
            logger.info("Total time for Calculate zscores and save wasting confusion matrix: %.2f", end - start)

        if SEX_IDX in self.data_config.TARGET_INDEXES:
            csv_fpath = f"{OUTPUT_CSV_PATH}/sex_evaluation_{descriptor}.csv"
            logger.info("Calculate and save sex results to %s", csv_fpath)
            calculate_and_save_results(df_grouped, eval_config.NAME, csv_fpath,
                                       self.data_config, result_config, fct=calculate_performance_sex)
        if GOODBAD_IDX in self.data_config.TARGET_INDEXES:
            csv_fpath = f"{OUTPUT_CSV_PATH}/goodbad_evaluation_{descriptor}.csv"
            logger.info("Calculate performance on bad/good scans and save results to %s", csv_fpath)
            calculate_and_save_results(df_grouped, eval_config.NAME, csv_fpath,
                                       self.data_config, result_config, fct=calculate_performance_goodbad)


class EnsembleEvaluation(Evaluation):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_the_model_path(self, workspace: Workspace):
        """Get multiple model paths"""
        for run_id in self.model_config.RUN_IDS:
            logger.info(f"Downloading run {run_id}")
            download_model(
                workspace=workspace,
                experiment_name=self.model_config.EXPERIMENT_NAME,
                run_id=run_id,
                input_location=self._input_location,
                output_location=self.model_base_dir / run_id
            )
        model_paths = glob.glob(os.path.join(self.model_base_dir, "*"))
        model_paths = [p for p in model_paths if os.path.isdir(p)]
        model_paths = [p for p in model_paths if p.split("/")[-1].startswith(self.model_config.EXPERIMENT_NAME)]
        model_paths = [os.path.join(p, self.model_config.INPUT_LOCATION, self.model_config.NAME) for p in model_paths]
        logger.info(f"Models paths ({len(model_paths)}):")
        logger.info("\t" + "\n\t".join(model_paths))
        self.model_paths = model_paths
        self.model_path_or_paths = model_paths

    def get_prediction_(self,
                        model_paths: List[Path],
                        dataset_evaluation: tf.data.Dataset) -> np.ndarray:
        return get_predictions_from_multiple_models(model_paths, dataset_evaluation, self.data_config)

    def evaluate(self,
                 df: pd.DataFrame,
                 result_config: Bunch,
                 eval_config: Bunch,
                 OUTPUT_CSV_PATH: str,
                 descriptor: str):
        super().evaluate(df, result_config, eval_config, OUTPUT_CSV_PATH, descriptor)

        if not result_config.USE_UNCERTAINTY:
            return

        # Sample one artifact per scan (qrcode, scantype combination)
        df_sample = df.groupby(['qrcode', 'scantype']).apply(lambda x: x.sample(1))

        # Prepare uncertainty prediction on these artifacts
        dataset_sample = prepare_sample_dataset(df_sample, self.dataset_path, self.data_config)

        # Predict uncertainty
        uncertainties = get_prediction_uncertainty_deepensemble(self.model_paths, dataset_sample)

        assert len(df_sample) == len(uncertainties)
        df_sample['uncertainties'] = uncertainties

        if GOODBAD_IDX in self.data_config.TARGET_INDEXES:
            assert COLUMN_NAME_GOODBAD in df
            png_fpath = f"{OUTPUT_CSV_PATH}/uncertainty_distribution.png"
            draw_uncertainty_goodbad_plot(df_sample, png_fpath)

            df_sample_100 = df_sample.iloc[df_sample.index.get_level_values('scantype') == '100']
            png_fpath = f"{OUTPUT_CSV_PATH}/uncertainty_code100_distribution.png"
            draw_uncertainty_goodbad_plot(df_sample_100, png_fpath)

        png_fpath = f"{OUTPUT_CSV_PATH}/uncertainty_scatter_distribution.png"
        draw_uncertainty_scatterplot(df_sample, png_fpath)

        # Filter for scans with high certainty and calculate their accuracy/results
        df_sample['error'] = df_sample.apply(avgerror, axis=1).abs()
        df_sample_better_threshold = df_sample[df_sample['uncertainties'] < result_config.UNCERTAINTY_THRESHOLD_IN_CM]
        csv_fpath = f"{OUTPUT_CSV_PATH}/uncertainty_smaller_than_{result_config.UNCERTAINTY_THRESHOLD_IN_CM}cm.csv"
        logger.info("Uncertainty: For more certain than %.2f cm, calculate and save the results to %s",
                    result_config.UNCERTAINTY_THRESHOLD_IN_CM, csv_fpath)
        calculate_and_save_results(df_sample_better_threshold, eval_config.NAME, csv_fpath,
                                   self.data_config, result_config, fct=calculate_performance)


class MultiartifactEvaluation(Evaluation):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def prepare_dataset(self,
                        qrcode_paths: List[str],
                        filter_config: Bunch) -> Tuple[tf.data.Dataset, List[List[str]]]:
        samples_paths = create_multiartifact_paths_for_qrcodes(qrcode_paths, self.data_config)

        depthmaps, targets = [], []
        for sample_paths in samples_paths:
            depthmap, target = create_multiartifact_sample(sample_paths,
                                                           self.data_config.NORMALIZATION_VALUE,
                                                           self.data_config.IMAGE_TARGET_HEIGHT,
                                                           self.data_config.IMAGE_TARGET_WIDTH,
                                                           tf.constant(self.data_config.TARGET_INDEXES),
                                                           self.data_config.N_ARTIFACTS)
            depthmaps.append(depthmap)
            targets.append(target)

        dataset = tf.data.Dataset.from_tensor_slices((depthmaps, targets))
        dataset = dataset.batch(self.data_config.BATCH_SIZE)

        return dataset, samples_paths

    def get_prediction_(self,
                        model_path: Path,
                        dataset_evaluation: List[str]) -> np.ndarray:
        predictions = get_prediction_multiartifact(model_path, dataset_evaluation, self.data_config)
        return predictions

    def prepare_dataframe(self,
                          paths_belonging_to_predictions: List[List[str]],
                          prediction_array: np.ndarray,
                          result_config: Bunch):
        first_paths = [paths_list[0] for paths_list in paths_belonging_to_predictions]
        qrcode_list, scantype_list, artifact_list, prediction_list, target_list = get_column_list(
            first_paths, prediction_array, self.data_config)

        df = pd.DataFrame({
            'qrcode': qrcode_list,
            'artifact': artifact_list,
            'scantype': scantype_list,
            'GT': target_list if target_list[0].shape == tuple() else [el[0] for el in target_list],
            'predicted': prediction_list
        }, columns=result_config.COLUMNS)
        df['GT'] = df['GT'].astype('float64')
        df['predicted'] = df['predicted'].astype('float64')

        if 'AGE_BUCKETS' in result_config.keys():
            idx = self.data_config.TARGET_INDEXES.index(AGE_IDX)
            df[COLUMN_NAME_AGE] = [el[idx] for el in target_list]
        if SEX_IDX in self.data_config.TARGET_INDEXES:
            idx = self.data_config.TARGET_INDEXES.index(SEX_IDX)
            df[COLUMN_NAME_SEX] = [el[idx] for el in target_list]
        if GOODBAD_IDX in self.data_config.TARGET_INDEXES:
            idx = self.data_config.TARGET_INDEXES.index(GOODBAD_IDX)
            df[COLUMN_NAME_GOODBAD] = [el[idx] for el in target_list]

        logger.info("df.shape: %s", df.shape)
        return df



from collections import namedtuple
from copy import copy
import logging
from typing import List, Tuple

from bunch import Bunch
import pandas as pd

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s - %(pathname)s: line %(lineno)d'))
logger.addHandler(handler)

IMAGE_TARGET_WIDTH = 180
IMAGE_TARGET_HEIGHT = 240
ORDER_DIFFERENCE_ALLOWED = 0  # works for scan version v0.9+

FusedArtifact = namedtuple('FusedArtifact', 'depth_artifact image_artifact')


class Artifact(Bunch):
    pass


def find_closest_image(image_order_numbers, depth_order_number):
    """Find corresponding image for the given depthmap on the basis of order"""
    closest_image_order_number = min(image_order_numbers, key=lambda image_order: abs(image_order - depth_order_number))
    return closest_image_order_number


def find_image_artifact_with_order_number(image_artifacts: List[Artifact], order_number: str) -> Artifact:
    """Find the image artifact with the given order_number"""
    for artifact in image_artifacts:
        if artifact.order_number == order_number:
            return artifact


def check_fields_are_equal(image_artifact: Artifact, depth_artifact: Artifact) -> None:
    """Check scan_id, height, weight, muac, scan_step, order_number are equal"""
    fields_supposed_to_be_equal = list(depth_artifact.keys())
    fields_to_ignore = ['file_path', 'timestamp', 'format']
    for f in fields_to_ignore:
        if f in fields_supposed_to_be_equal:
            fields_supposed_to_be_equal.remove(f)

    for field in fields_supposed_to_be_equal:
        depth_artifact_value = getattr(depth_artifact, field)
        image_artifact_value = getattr(image_artifact, field)
        if depth_artifact_value != image_artifact_value:
            raise ValueError(f"{field} is not equal for depthmap and image artifact")


def match_depth_and_image_artifacts(image_artifacts: List[Artifact],
                                    depth_artifacts: List[Artifact]) -> List[FusedArtifact]:
    fused_artifacts = []
    image_order_numbers = [image_artifact.order_number for image_artifact in image_artifacts]

    for depth_artifact in depth_artifacts:
        closest_image_order_number = find_closest_image(image_order_numbers, depth_artifact.order_number)
        if abs(closest_image_order_number - depth_artifact.order_number) > ORDER_DIFFERENCE_ALLOWED:
            logger.debug(f"No corresponding image found for depthmap with order_number {depth_artifact.order_number}")
            continue
        image_artifact = find_image_artifact_with_order_number(image_artifacts, closest_image_order_number)
        check_fields_are_equal(image_artifact, depth_artifact)
        fused_artifacts.append(FusedArtifact(depth_artifact, image_artifact))
    return fused_artifacts


def fused_artifact2dict(fused_artifact: FusedArtifact) -> dict:
    out_fused_artifact = copy(fused_artifact.depth_artifact)
    image_artifact = fused_artifact.image_artifact

    out_fused_artifact['file_path_rgb'] = image_artifact.file_path
    return dict(out_fused_artifact)


def match_df_with_depth_and_image_artifacts(df: pd.DataFrame) -> List[dict]:
    """Process a dataframe with the depthmap and the corresponding image"""
    all_fused_artifacts = []
    scan_ids = df.scan_id.unique()

    for scan_id in scan_ids:
        df_scan = df[df.scan_id == scan_id]
        assert len(df_scan.scan_step.unique()) == 1

        df_depth = df_scan.loc[df.format == 'depth']
        df_image = df_scan.loc[df.format == 'rgb']

        depths: List[Tuple[str]] = list(df_depth.itertuples(index=False, name=None))
        images: List[Tuple[str]] = list(df_image.itertuples(index=False, name=None))

        depth_artifacts = [Artifact(zip(df.columns, d)) for d in depths]
        image_artifacts = [Artifact(zip(df.columns, i)) for i in images]

        fused_artifacts = match_depth_and_image_artifacts(image_artifacts, depth_artifacts)
        fused_artifacts_dicts = [fused_artifact2dict(a) for a in fused_artifacts]
        all_fused_artifacts.extend(fused_artifacts_dicts)
    return all_fused_artifacts



import os

from bunch import Bunch

CONFIG_NAME = os.path.splitext(os.path.basename(__file__))[0]

# Details of model used for evaluation
MODEL_CONFIG = Bunch(dict(
    EXPERIMENT_NAME='q3-depthmap-plaincnn-height-95k',
    RUN_ID='q3-depthmap-plaincnn-height-95k_1617983539_763a24b9',  # Run 9
    INPUT_LOCATION='outputs',
    NAME='best_model.ckpt',
))


EVAL_CONFIG = Bunch(dict(
    # Name of evaluation
    NAME='q3-depthmap-plaincnn-height-95k-run_09',

    # Experiment in Azure ML which will be used for evaluation
    EXPERIMENT_NAME="QA-pipeline",
    CLUSTER_NAME="gpu-cluster",

    # Used for Debug the QA pipeline
    DEBUG_RUN=False,

    # Will run eval on specified # of scan instead of full dataset
    DEBUG_NUMBER_OF_SCAN=5,

    SPLIT_SEED=0,
))

#Details of Evaluation Dataset
DATA_CONFIG = Bunch(dict(
    NAME='anon-depthmap-testset',  # Name of evaluation dataset

    IMAGE_TARGET_HEIGHT=240,
    IMAGE_TARGET_WIDTH=180,

    BATCH_SIZE=512,  # Batch size for evaluation
    NORMALIZATION_VALUE=7.5,

    TARGET_INDEXES=[0],  # 0 is height, 1 is weight
    CODES=['100', '101', '102', '200', '201', '202'],
))


# Result configuration for result generation after evaluation is done
RESULT_CONFIG = Bunch(dict(
    # Error margin on various ranges
    ACCURACIES=[.2, .4, .6, 1., 1.2, 2., 2.5, 3., 4., 5., 6.],  # 0.2cm, 0.4cm, 0.6cm, 1cm, ...
    ACCURACY_MAIN_THRESH=1.0,  # 1cm

    COLUMNS=['qrcode', 'artifact', 'scantype', 'GT', 'predicted'],

    # uncertainty
    USE_UNCERTAINTY=False,  # Flag to enable model uncertainty calculation

    # path of csv file in the experiment which final result is stored
    SAVE_PATH=f'./outputs/{CONFIG_NAME}',
))



import pickle
import pandas as pd
import numpy as np
from pyntcloud import PyntCloud
import pathlib
import os
import logging
import logging.config

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s - %(pathname)s: line %(lineno)d'))
logger.addHandler(handler)


source_path = '/mnt/depthmap/pcd_50k/'
target_path = '/mnt/depthmap/pcd_50k/pcd_50k_pkl'
dataset = '../../gitrepo/cgm-ml-service/dataset_EDA/opensource_pcd_data/artifacts.csv'

if not os.path.exists(target_path):
    os.mkdir(target_path)


def load_pcd_as_ndarray(pcd_path):
    """
    Loads a PCD-file. Yields a numpy-array.
    """
    pointcloud = PyntCloud.from_file(pcd_path)
    values = pointcloud.points.values
    return values


def pcd_to_pickle(dataset):
    """
    --dataset: A csv file conataining height, weight, qrcode, storage path for pcd

    return:
    -- folder with pickle file having pointcloud, height and weight
    """
    data = pd.read_csv(dataset)
    for index, rows in data.iterrows():
        pcd_path = source_path + rows['storage_path']
        pointcloud = load_pcd_as_ndarray(pcd_path)
        label = np.array([float(rows['height']), float(rows['weight'])])
        qrcode_path = os.path.join(target_path, rows['qrcode'])
        pickle_filename = os.path.basename(rows['artifacts']).replace(".pcd", ".p")
        pathlib.Path(qrcode_path).mkdir(parents=True, exist_ok=True)
        pickle_output_path = os.path.join(qrcode_path, pickle_filename)
        pickle.dump((pointcloud, label), open(pickle_output_path, "wb"))


def main():
    logger.info("Starting the data preparation.")
    pcd_to_pickle(dataset)
    logger.info("dataset preparation finished.Check the %s path for outputs", target_path)


if __name__ == "__main__":
    main()



import os

from bunch import Bunch

CONFIG_NAME = os.path.splitext(os.path.basename(__file__))[0]

# Details of model used for evaluation
MODEL_CONFIG = Bunch(dict(
    EXPERIMENT_NAME='q4-depthmap-plaincnn-weight-95k',
    RUN_ID='q4-depthmap-plaincnn-weight-95k_1611336518_642a9c58',  # Run 2
    INPUT_LOCATION='outputs',
    NAME='best_model.ckpt',
))


EVAL_CONFIG = Bunch(dict(
    # Name of evaluation
    NAME='q4-depthmap-plaincnn-weight-95k_run_02',

    # Experiment in Azure ML which will be used for evaluation
    EXPERIMENT_NAME="QA-pipeline",
    CLUSTER_NAME="gpu-cluster",

    # Used for Debug the QA pipeline
    DEBUG_RUN=False,

    # Will run eval on specified # of scan instead of full dataset
    DEBUG_NUMBER_OF_SCAN=5,

    SPLIT_SEED=0,
))

# Details of Evaluation Dataset
DATA_CONFIG = Bunch(dict(
    NAME='anon-realtime-testdata',  # Name of evaluation dataset

    IMAGE_TARGET_HEIGHT=240,
    IMAGE_TARGET_WIDTH=180,

    BATCH_SIZE=512,  # Batch size for evaluation
    NORMALIZATION_VALUE=7.5,

    # 0 is height, 1 is weight, 2 is muac, 3 is age,
    # 4 is sex('male' or 'female'), 5 is quality ('good' or 'bad'), 6 is test
    TARGET_INDEXES=[1, 3, 4, 5],

    CODES=['100', '101', '102', '200', '201', '202']
))


#Result configuration for result generation after evaluation is done
RESULT_CONFIG = Bunch(dict(
    # Error margin on various ranges
    ACCURACIES=[0.04, 0.1, 0.21, 0.42],  # 40gms, 100gms, 210gms, 420gms
    ACCURACY_MAIN_THRESH=0.21,  # 210gms
    AGE_BUCKETS=[0, 1, 2, 3, 4, 5],

    COLUMNS=['qrcode', 'artifact', 'scantype', 'GT', 'predicted'],

    # uncertainty
    USE_UNCERTAINTY=False,  # Flag to enable model uncertainty calculation

    # path of csv file in the experiment which final result is stored
    SAVE_PATH=f'./outputs/{CONFIG_NAME}',
))



import logging
import logging.config
from pathlib import Path
import shutil

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s - %(pathname)s: line %(lineno)d'))
logger.addHandler(handler)


def copy_dir(src: Path, tgt: Path, glob_pattern: str, should_touch_init: bool = False):
    logger.info("Creating temp folder")
    if tgt.exists():
        shutil.rmtree(tgt)
    tgt.mkdir(parents=True, exist_ok=True)
    if should_touch_init:
        (tgt / '__init__.py').touch(exist_ok=False)

    paths_to_copy = list(src.glob(glob_pattern))
    logger.info(f"Copying to {tgt} the following files: {str(paths_to_copy)}")
    for p in paths_to_copy:
        destpath = tgt / p.relative_to(src)
        destpath.parent.mkdir(parents=True, exist_ok=True)
        shutil.copy(p, destpath)



#
# Child Growth Monitor - Free Software for Zero Hunger
# Copyright (c) 2019 Tristan Behrens <tristan@ai-guru.de> for Welthungerhilfe
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


"""
Helper module for Neural Networks.
"""

from tensorflow.keras import models, layers
import numpy as np
import tensorflow as tf
import os
import logging
import logging.config
import pickle

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s - %(pathname)s: line %(lineno)d'))
logger.addHandler(handler)


def create_multiview_model(base_model, multiviews_num, input_shape, output_size, use_lstm):

    assert base_model == "voxnet" or base_model == "pointnet"

    if base_model == "voxnet":
        base_model = create_voxnet_model_homepage(input_shape, output_size)
    elif base_model == "pointnet":
        base_model = create_point_net(input_shape, output_size)

    input = layers.Input(shape=(multiviews_num,) + input_shape)

    multiview_outputs = []
    for i in range(multiviews_num):
        multiview_input = layers.Lambda(lambda x: x[:, i])(input)
        multiview_output = base_model(multiview_input)
        multiview_outputs.append(multiview_output)
    output = layers.Average()(multiview_outputs)

    model = models.Model(input, output)
    return model


def create_multiview_model_old(base_model, multiviews_num, input_shape, output_size, use_lstm):

    assert base_model == "voxnet" or base_model == "pointnet"

    if base_model == "voxnet":
        base_model = create_voxnet_model_homepage(input_shape, output_size)
    elif base_model == "pointnet":
        base_model = create_point_net(input_shape, output_size)

    model = models.Sequential()
    model.add(layers.TimeDistributed(base_model, input_shape=(multiviews_num,) + input_shape))
    if use_lstm is True:
        model.add(layers.LSTM(8, activation="relu"))
    else:
        model.add(layers.AveragePooling1D(multiviews_num))
        model.add(layers.Flatten())
    model.add(layers.Dense(output_size))

    return model


def create_dense_model(input_shape, output_size):
    """
    Creates a simple dense model.

    Note: This is only suitable for baseling.

    Args:
        input_shape (shape): Input-shape.
        output_size (int): Output-size.

    Returns:
        Model: A model.
    """

    model = models.Sequential(name="baseline-dense")
    model.add(layers.Flatten(input_shape=input_shape))
    model.add(layers.Dense(128, activation="relu"))
    model.add(layers.Dense(64, activation="relu"))
    model.add(layers.Dense(output_size))

    return model


def create_voxnet_model_small(input_shape, output_size):
    """
    Creates a small VoxNet.

    See: http://dimatura.net/publications/3dcnn_lz_maturana_scherer_icra15.pdf

    Args:
        input_shape (shape): Input-shape.
        output_size (int): Output-size.

    Returns:
        Model: A model.
    """

    #Trainable params: 301,378
    model = models.Sequential(name="C7-F32-P2-C5-F64-P2-D512")
    model.add(layers.Reshape(target_shape=input_shape + (1,), input_shape=input_shape))
    model.add(layers.Conv3D(32, (7, 7, 7), activation="relu"))
    model.add(layers.MaxPooling3D((4, 4, 4)))
    model.add(layers.Conv3D(64, (5, 5, 5), activation="relu"))
    model.add(layers.MaxPooling3D((2, 2, 2)))
    model.add(layers.Flatten())
    model.add(layers.Dense(512, activation="relu"))
    model.add(layers.Dense(output_size))

    return model


def create_voxnet_model_big(input_shape, output_size):
    """
    Creates a big VoxNet.

    See: http://dimatura.net/publications/3dcnn_lz_maturana_scherer_icra15.pdf

    Args:
        input_shape (shape): Input-shape.
        output_size (int): Output-size.

    Returns:
        Model: A model.
    """

    # Trainable params: 7,101,442
    model = models.Sequential(name="C7-F64-P4-D512")
    model.add(layers.Reshape(target_shape=input_shape + (1,), input_shape=input_shape))
    model.add(layers.Conv3D(64, (7, 7, 7), activation="relu"))
    model.add(layers.MaxPooling3D((4, 4, 4)))
    model.add(layers.Flatten())
    model.add(layers.Dense(512, activation="relu"))
    model.add(layers.Dense(output_size))

    return model


def create_voxnet_model_homepage(input_shape, output_size):
    """
    Creates a small VoxNet.

    See: http://dimatura.net/publications/3dcnn_lz_maturana_scherer_icra15.pdf

    Note: This is the latest model that the VoxNet-authors used.

    Args:
        input_shape (shape): Input-shape.
        output_size (int): Output-size.

    Returns:
        Model: A model.
    """

    # Trainable params: 916,834
    model = models.Sequential(name="VoxNetHomepage")
    model.add(layers.Reshape(target_shape=input_shape + (1,), input_shape=input_shape))
    model.add(layers.Conv3D(32, (5, 5, 5), strides=(2, 2, 2), activation="relu"))
    model.add(layers.Conv3D(32, (3, 3, 3), strides=(1, 1, 1), activation="relu"))
    model.add(layers.MaxPooling3D((2, 2, 2)))
    model.add(layers.Flatten())
    model.add(layers.Dense(128, activation="relu"))
    model.add(layers.Dense(output_size))

    return model


def create_point_net(input_shape, output_size, hidden_sizes=[512, 256], use_lambda=False):
    """
    Creates a PointNet.

    See https://github.com/garyloveavocado/pointnet-keras/blob/master/train_cls.py

    Args:
        input_shape (shape): Input-shape.
        output_size (int): Output-size.

    Returns:
        Model: A model.
    """

    logger.info('Input Shape: %s', str(input_shape))

    num_points = input_shape[0]

    def mat_mul(A, B):
        result = tf.matmul(A, B)
        return result

    input_points = layers.Input(shape=input_shape)
    x = layers.Convolution1D(64, 1, activation='relu', input_shape=input_shape)(input_points)
    x = layers.BatchNormalization()(x)
    x = layers.Convolution1D(128, 1, activation='relu')(x)
    x = layers.BatchNormalization()(x)
    x = layers.Convolution1D(1024, 1, activation='relu')(x)
    x = layers.BatchNormalization()(x)
    x = layers.MaxPooling1D(pool_size=num_points)(x)
    x = layers.Dense(512, activation='relu')(x)
    x = layers.BatchNormalization()(x)
    x = layers.Dense(256, activation='relu')(x)
    x = layers.BatchNormalization()(x)
    x = layers.Dense(9, weights=[np.zeros([256, 9]), np.array(
        [1, 0, 0, 0, 1, 0, 0, 0, 1]).astype(np.float32)])(x)
    input_T = layers.Reshape((input_shape[1], input_shape[1]))(x)

    # forward net
    if use_lambda is True:
        g = layers.Lambda(mat_mul, arguments={'B': input_T})(input_points)
    else:
        g = layers.dot([input_points, input_T], axes=-1, normalize=True)
    g = layers.Convolution1D(64, 1, input_shape=input_shape, activation='relu')(input_points)
    g = layers.BatchNormalization()(g)
    g = layers.Convolution1D(64, 1, input_shape=input_shape, activation='relu')(g)
    g = layers.BatchNormalization()(g)

    # feature transform net
    f = layers.Convolution1D(64, 1, activation='relu')(g)
    f = layers.BatchNormalization()(f)
    f = layers.Convolution1D(128, 1, activation='relu')(f)
    f = layers.BatchNormalization()(f)
    f = layers.Convolution1D(1024, 1, activation='relu')(f)
    f = layers.BatchNormalization()(f)
    f = layers.MaxPooling1D(pool_size=num_points)(f)
    f = layers.Dense(512, activation='relu')(f)
    f = layers.BatchNormalization()(f)
    f = layers.Dense(256, activation='relu')(f)
    f = layers.BatchNormalization()(f)
    f = layers.Dense(64 * 64, weights=[np.zeros([256, 64 * 64]),
                                       np.eye(64).flatten().astype(np.float32)])(f)
    feature_T = layers.Reshape((64, 64))(f)

    # forward net
    if use_lambda is True:
        g = layers.Lambda(mat_mul, arguments={'B': feature_T})(g)
    else:
        g = layers.dot([g, feature_T], axes=-1, normalize=True)
    g = layers.Convolution1D(64, 1, activation='relu')(g)
    g = layers.BatchNormalization()(g)
    g = layers.Convolution1D(128, 1, activation='relu')(g)
    g = layers.BatchNormalization()(g)
    g = layers.Convolution1D(1024, 1, activation='relu')(g)
    g = layers.BatchNormalization()(g)

    # global_feature
    global_feature = layers.MaxPooling1D(pool_size=num_points)(g)

    # point_net_cls
    c = global_feature
    for hidden_size in hidden_sizes:
        c = layers.Dense(hidden_size, activation='relu')(c)
        c = layers.BatchNormalization()(c)
        c = layers.Dropout(rate=0.3)(c)

    c = layers.Dense(output_size, activation='linear')(c)
    prediction = layers.Flatten()(c)

    model = models.Model(inputs=input_points, outputs=prediction)
    return model


def create_dense_net(input_shape, output_size, hidden_sizes=[]):

    model = models.Sequential()

    # Input layer.
    model.add(layers.Flatten(input_shape=input_shape))

    for hidden_size in hidden_sizes:
        model.add(layers.Dense(hidden_size, activation="relu"))

    # Output layer.
    model.add(layers.Dense(output_size))

    return model


def create_2dCNN(input_shape, output_size):
    """
    Creates a 2dCNN.

    Args:
        input_shape (shape): Input-shape.
        output_size (int): Output-size.

    Returns:
        Model: A model.
    """

    # Trainable params: 13,086,913

    model_cnn = models.Sequential(name="2dCNN")
    model_cnn.add(layers.Conv2D(
        32, (3, 3),
        activation="relu",
        input_shape=(input_shape)))
    model_cnn.add(layers.MaxPooling2D((2, 2)))
    model_cnn.add(layers.Conv2D(64, (3, 3), activation="relu"))
    model_cnn.add(layers.MaxPooling2D((2, 2)))
    model_cnn.add(layers.Conv2D(128, (3, 3), activation="relu"))
    model_cnn.add(layers.MaxPooling2D((2, 2)))
    model_cnn.add(layers.Conv2D(128, (3, 3), activation="relu"))
    model_cnn.add(layers.MaxPooling2D((2, 2)))
    model_cnn.add(layers.Flatten())
    model_cnn.add(layers.Dense(512, activation="relu"))
    model_cnn.add(layers.Dropout(0.25))
    model_cnn.add(layers.Dense(output_size, activation="relu"))

    return model_cnn


def create_vgg(input_shape, output_size):
    """
   Creates a vgg19 and add Dense Layers.

   Args:
       input_shape (shape): Input-shape.
       output_size (int): Output-size.

   Returns:
       Model: A model.
   """

    # Trainable params: 68,229,825
    # Non-trainable params: 20,024,384
#     vgg19 = applications.VGG19(include_top=False, input_shape=(256, 256, 3))
#     vgg19.trainable = False
#     model_vgg = models.Sequential()
#     model_vgg.add(vgg19)
#     model_vgg.add(layers.Flatten())
#     model_vgg.add(layers.Dense(2048, activation='relu'))
#     model_vgg.add(layers.Dense(512, activation='relu'))
#     model_vgg.add(layers.Dense(128, activation='relu'))
#     model_vgg.add(layers.Dense(32, activation='relu'))
#     model_vgg.add(layers.Dense(1, activation='relu')) #regression

#     return model_vgg


# Method for saving model and history.
def save_model_and_history(output_path, datetime_string, model, history, training_details, name):

    logger.info("Saving model and history...")

    # Try to save model. Could fail.
    try:
        model_name = datetime_string + "-" + name + "-model.h5"
        model_path = os.path.join(output_path, model_name)
        model.save(model_path)
        logger.info("Saved model to %s", model_path)
    except Exception:
        logger.info("WARNING! Failed to save model. Use model-weights instead.")
        pass

    # Save the model weights.
    model_weights_name = datetime_string + "-" + name + "-model-weights.h5"
    model_weights_path = os.path.join(output_path, model_weights_name)
    model.save_weights(model_weights_path)
    logger.info("Saved model weights to %s", model_weights_path)

    # Save the training details.
    training_details_name = datetime_string + "-" + name + "-details.p"
    training_details_path = os.path.join(output_path, training_details_name)
    pickle.dump(training_details, open(training_details_path, "wb"))
    logger.info("Saved training details to %s", training_details_path)

    # Save the history.
    history_name = datetime_string + "-" + name + "-history.p"
    history_path = os.path.join(output_path, history_name)
    pickle.dump(history.history, open(history_path, "wb"))
    logger.info("Saved history to %s", history_path)


def load_pointnet(weights_path, input_shape, output_size, hidden_sizes):
    try:
        model = create_point_net(input_shape, output_size, hidden_sizes, use_lambda=False)
        model.load_weights(weights_path)
    except Exception:
        model = create_point_net(input_shape, output_size, hidden_sizes, use_lambda=True)
        model.load_weights(weights_path)
        return model



import json
from pathlib import Path


class Table:
    def __init__(self, table_file: Path):
        self.table_file = table_file

    def load_table(self) -> None:
        table_file = open(self.table_file, 'r')
        self.table_value = json.load(table_file)

    def add_value(self) -> None:
        if 'Length' in self.table_value[0]:
            field_name = 'Length'
        elif 'Height' in self.table_value[0]:
            field_name = 'Height'
        elif 'Day' in self.table_value[0]:
            field_name = 'Day'
        else:
            raise Exception('error loading: %s' % self.table_value)
        self.field_name_dict = {'field_name': field_name}

    def append_value(self) -> dict:
        for value in self.table_value:
            self.field_name_dict.update({value[self.field_name_dict['field_name']]: value})
        return self.field_name_dict



# examples/Python/Basic/pointcloud.py

import logging
import logging.config
import numpy as np
import open3d

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(asctime)s - %(levelname)s - %(message)s - %(pathname)s: line %(lineno)d'))
logger.addHandler(handler)

ENABLE_VISUALIZATION = False
DOWNSAMPLE = True

if __name__ == "__main__":

    logger.info("Load a ply point cloud, print it, and render it")
    pcd = open3d.io.read_point_cloud("/data/home/cpfitzner/test.pcd")
    logger.info(pcd)
    logger.info(np.asarray(pcd.points))
    if ENABLE_VISUALIZATION:
        open3d.visualization.draw_geometries([pcd])

    downpcd = pcd
    if DOWNSAMPLE:
        logger.info("DOWNSAMPLE the point cloud with a voxel of 0.05")
        downpcd = pcd.voxel_down_sample(voxel_size=0.05)
        logger.info(downpcd)
        logger.info(np.asarray(downpcd.points))

    if ENABLE_VISUALIZATION:
        open3d.visualization.draw_geometries([downpcd])

    logger.info("Recompute the normal of the DOWNSAMPLEd point cloud")
    downpcd.estimate_normals(search_param=open3d.geometry.KDTreeSearchParamHybrid(
        radius=0.1, max_nn=30))
    if ENABLE_VISUALIZATION:
        open3d.visualization.draw_geometries([downpcd])

    logger.info("Print a normal vector of the 0th point")
    logger.info(downpcd.normals[0])
    logger.info("Print the normal vectors of the first 10 points")
    logger.info(np.asarray(downpcd.normals)[:10, :])
    logger.info("x: ")
    logger.info(np.asarray(downpcd.normals)[0, 0])
    logger.info("y: ")
    logger.info(np.asarray(downpcd.normals)[0, 1])
    logger.info("z: ")
    logger.info(np.asarray(downpcd.normals)[0, 2])
    logger.info("")

    logger.info("Load a polygon volume and use it to crop the original point cloud")
    vol = open3d.visualization.read_selection_polygon_volume(
        "../../TestData/Crop/cropped.json")
    chair = vol.crop_point_cloud(pcd)
    if ENABLE_VISUALIZATION:
        open3d.visualization.draw_geometries([chair])
        logger.info("")

        logger.info("Paint chair")
        chair.paint_uniform_color([1, 0.706, 0])
        open3d.visualization.draw_geometries([chair])
    logger.info("")



from bunch import Bunch

# Error margin on various ranges
EVALUATION_ACCURACIES = [.2, .4, .6, 1., 1.2, 2., 2.5, 3., 4., 5., 6.]
MODEL_CKPT_FILENAME = "best_model.ckpt"

DAYS_IN_YEAR = 365

HEIGHT_IDX = 0
WEIGHT_IDX = 1
MUAC_IDX = 2
AGE_IDX = 3
SEX_IDX = 4
GOODBAD_IDX = 5

SEX_DICT = {'female': 0., 'male': 1.}
GOODBAD_DICT = {'bad': 0., 'good': 1., 'delete': 2.}

COLUMN_NAME_AGE = 'GT_age'
COLUMN_NAME_SEX = 'GT_sex'
COLUMN_NAME_GOODBAD = 'GT_goodbad'
CODE_TO_SCANTYPE = {
    '100': '_standingfront',
    '101': '_standing360',
    '102': '_standingback',
    '200': '_lyingfront',
    '201': '_lyingrot',
    '202': '_lyingback',
}

DATA_AUGMENTATION_SAME_PER_CHANNEL = "same_per_channel"
DATA_AUGMENTATION_DIFFERENT_EACH_CHANNEL = "different_each_channel"
DATA_AUGMENTATION_NO = "no"

SAMPLING_STRATEGY_SYSTEMATIC = "systematic"
SAMPLING_STRATEGY_WINDOW = "window"

CONFIG = Bunch(dict(
    IMAGE_TARGET_HEIGHT=240,
    IMAGE_TARGET_WIDTH=180,
    NORMALIZATION_VALUE=7.5,
    TARGET_INDEXES=[0],  # 0 is height, 1 is weight.
    DATA_AUGMENTATION_MODE=DATA_AUGMENTATION_NO,
    SAMPLING_STRATEGY=SAMPLING_STRATEGY_SYSTEMATIC,
    N_ARTIFACTS=5,
    N_REPEAT_DATASET=1,
    CODES=('100', '101', '102', '200', '201', '202'),
))
